# Copyright (c) 2015 Riverbed Technology, Inc.
#
# This software is licensed under the terms and conditions of the MIT License
# accompanying the software ("License").  This software is distributed "AS IS"
# as set forth in the License.

import os
import re
import cPickle
import logging
import subprocess
import datetime
import tempfile
import pytz
import tzlocal

from dateutil.parser import parse as dateutil_parse

from steelscript.common.timeutils import parse_timedelta
from steelscript.wireshark.core.exceptions import InvalidField


logger = logging.getLogger(__name__)

local_tz = tzlocal.get_localzone()


class PcapFile(object):

    OCCURRENCE_FIRST = 'f'
    OCCURRENCE_LAST = 'l'
    OCCURRENCE_ALL = 'a'

    def __init__(self, filename):
        self.filename = filename

        self._info = None
        self.starttime = None
        self.endtime = None

    def info(self):
        """Returns info on pcap file, uses ``capinfos -A -m -T`` internally"""
        if self._info is None:

            cmd = ['capinfos', '-A', '-m', '-T', self.filename]
            logger.info('subprocess: %s' % ' '.join(cmd))
            capinfos = subprocess.check_output(cmd)
            hdrs, vals = (capinfos.split('\n')[:2])
            self._info = dict(zip(hdrs.split(','), vals.split(',')))

            self.starttime = (dateutil_parse(self._info['Start time'])
                              .replace(tzinfo=local_tz))
            self.endtime = (dateutil_parse(self._info['End time'])
                            .replace(tzinfo=local_tz))

            self.numpackets = int(self._info['Number of packets'])

        return self._info

    def export(self, filename,
               starttime=None, endtime=None, duration=None):
        """Returns a PCAP file, potentially including your specified starttime,
        endtime or duration. Internally uses editcap

        :param str filename: the name of the new PCAP file to be
            created/exported from the existing PCAP file
        :param str starttime: defines a start time filter
        :param str endtime: defines an end time filter
        :param str duration: defines a duration filter

        """

        cmd = ['editcap']

        if starttime is not None:
            if isinstance(starttime, basestring):
                starttime = dateutil_parse(starttime)

        if endtime is not None:
            if isinstance(endtime, basestring):
                endtime = dateutil_parse(endtime)

        if duration is not None:
            if isinstance(duration, basestring):
                duration = parse_timedelta(duration)

            if starttime:
                endtime = starttime + duration
            elif endtime:
                starttime = endtime - duration
            else:
                raise ValueError("Must specify either starttime or "
                                 "endtime with duration")

        if starttime is not None:
            cmd.extend(['-A', (starttime
                               .strftime('%Y-%m-%d %H:%M:%S'))])

        if endtime is not None:
            cmd.extend(['-B', (endtime
                               .strftime('%Y-%m-%d %H:%M:%S'))])

        cmd.append(self.filename)
        cmd.append(filename)

        logger.info('subprocess: %s' % ' '.join(cmd))
        subprocess.check_output(cmd)

        return PcapFile(filename)

    def delete(self):
        """Removes the filename from PcapFile object and deletes the file"""
        if os.path.exists(self.filename):
            os.unlink(self.filename)

        self.filename = None

    def query(self, fieldnames, filterexpr=None,
              starttime=None, endtime=None, duration=None,
              use_tshark_fields=True,
              occurrence=OCCURRENCE_ALL,
              aggregator=',',
              as_dataframe=False):
        """Parses the PCAP file, returning the data in a tabular format.
        NOTE: When using OCCURRENCE_ALL you can generate an exception if there
        are multiple fields that have multiple values.

        :param str fieldnames: a list of field names for the desired values.
            Use the aggregator string for seperating columns
        :param str filterexpr: the filter expression used by tshark for
            refining your data
        :param str starttime: defines a start time filter for the query
        :param str endtime: defines an end time filter for the query
        :param str duration: defines a duration filter for the query
        :param bool use_tshark_fields: use the internal class TSharkField for
            more flexibility during data manipulation. Defaults to True.
        :param str occurrence: defines if you want the first, last, or all
            occurrences. Values:

                - 'f' (PcapFile.OCCURRENCE_FIRST) - First
                - 'l' (PcapFile.OCCURRENCE_LAST) - Last
                - 'a' (PcapFile.OCCURRENCE_ALL) - All

        :param str aggregator: delimiter string for seperating columns (aka
            tshark fields)
        :param bool as_dataframe: if true, returns a Pandas dataframe object
            with the results of the query. If false returns a list. Defaults
            to false.
        """
        if not self.filename:
            raise ValueError('No filename')

        cmd = ['tshark', '-r', self.filename,
               '-T', 'fields',
               '-E', 'occurrence=%s' % occurrence]

        if occurrence == self.OCCURRENCE_ALL:
            cmd.extend(['-E', 'aggregator=%s' % aggregator])

        if starttime or endtime:
            logger.info("Creating temp pcap file for timerange: %s-%s" %
                        (starttime, endtime))
            (fd, filename) = tempfile.mkstemp(suffix='.pcap')
            os.close(fd)
            p = self.export(filename,
                            starttime=starttime,
                            endtime=endtime,
                            duration=duration)
            logger.info("Issuing query on temp pcap file")
            res = p.query(fieldnames, filterexpr=filterexpr,
                          use_tshark_fields=use_tshark_fields,
                          occurrence=occurrence,
                          as_dataframe=as_dataframe)
            p.delete()
            return res

        if filterexpr not in [None, '']:
            # use new '-Y' option since '-R' is deprecated
            cmd.extend(['-Y', filterexpr])

        fields = []
        for n in fieldnames:
            if use_tshark_fields:
                tf = TSharkFields.instance()
                if n in tf.protocols:
                    # Allow protocols as a field, but convert to a string
                    # rather than attempt to parse it
                    fields.append(TSharkField(n, '', 'FT_STRING', n))

                elif n in tf.fields:
                    fields.append(tf.fields[n])

                else:
                    raise InvalidField(n)

            cmd.extend(['-e', n])

        logger.info('subprocess: %s' % ' '.join(cmd))
        proc = subprocess.Popen(cmd, stdout=subprocess.PIPE)

        data = []
        errors = 0
        while proc.poll() is None:
            line = proc.stdout.readline().rstrip()
            if not line:
                continue
            cols = line.split('\t')
            if len(cols) < len(fieldnames):
                cols.extend([None]*(len(fieldnames) - len(cols)))
            elif len(cols) > len(fieldnames):
                logger.error("Could not parse line: '%s'" % line)
                errors = errors + 1
                if errors > 20:
                    return
                continue

            if occurrence == PcapFile.OCCURRENCE_ALL:
                newcols = []
                needs_dup = []
                n = 0
                multi_occur = False
                for i, col in enumerate(cols):
                    if col and ',' in col:
                        if n:
                            logger.warning('One packet has at least '
                                           'two columns with multiple '
                                           'occurrences, skip it. '
                                           'cmd: %s' % ' '.join(cmd))
                            multi_occur = True
                            break
                        # Split col data into an array
                        newcol = col.split(',')
                        newcols.append(newcol)
                        n = len(newcol)
                    else:
                        # Single valued column, keep track of
                        # the col index, as we need to dup it
                        # below
                        newcols.append(col)
                        needs_dup.append(i)

                if multi_occur:
                    # The above for loop exited due to multiple occurrences of
                    # at least two columns in the current packet. Skip this
                    # packet and keep processing the rest of the pcap file
                    continue

                if n:
                    for i in needs_dup:
                        newcols[i] = ([newcols[i]] * n)
                    rows = (map(list, zip(*newcols)))
                else:
                    rows = [newcols]
            else:
                rows = [cols]

            if use_tshark_fields:
                newrows = []
                for row in rows:
                    newcols = []
                    for i, col in enumerate(row):
                        t = fields[i].datatype
                        if col == '' or col is None:
                            col = None
                        elif t == datetime.datetime:
                            col = (dateutil_parse(col)
                                   .replace(tzinfo=local_tz))
                        elif fields[i].name == 'frame.time_epoch':
                            col = (datetime.datetime.utcfromtimestamp(float(col))
                                   .replace(tzinfo=pytz.utc)
                                   .astimezone(local_tz))
                        elif t in [int, long]:
                            col = t(col, base=0)
                        else:
                            col = t(col)
                        newcols.append(col)
                    newrows.append(newcols)
                rows = newrows

            data.extend(rows)

        if as_dataframe:
            if len(data) > 0:
                import pandas
                df = pandas.DataFrame(data, columns=fieldnames)
                return df
            else:
                return None
        else:
            return data


class TSharkField(object):

    __slots__ = ['name', 'desc', 'datatype', 'datatype_str', 'protocol']

    def __init__(self, name, desc, datatype, protocol):
        self.name = name
        self.desc = desc
        self.protocol = protocol
        self.datatype_str = datatype

        if re.match('FT_U?INT64.*', datatype):
            self.datatype = long
        elif re.match('FT_(U?INT.*|FRAMENUM)', datatype):
            self.datatype = int
        elif re.match('FT_(FLOAT|DOUBLE|RELATIVE_TIME)', datatype):
            self.datatype = float
        elif re.match('FT_ABSOLUTE_TIME', datatype):
            self.datatype = datetime.datetime
        else:
            self.datatype = str

    def __str__(self):
        return '<TSharkField %s, %s>' % (self.name, self.datatype_str)

    def __repr__(self):
        return str(self)

    def __getstate__(self):
        return [self.name, self.desc, self.datatype_str,
                self.datatype, self.protocol]

    def __setstate__(self, state):
        (self.name, self.desc, self.datatype_str,
         self.datatype, self.protocol) = state


class TSharkFields(object):

    CACHEFILE = os.path.join(os.path.expanduser('~'), '.steelscript',
                             'tshark_fields')
    CACHEFILE_VERSION = 1

    _instance = None

    @classmethod
    def instance(cls):
        if cls._instance is None:
            cls._instance = TSharkFields()
        return cls._instance

    def __init__(self):
        self.protocols = None
        self.fields = None
        self.load()

    def load(self, force=False, ignore_cache=False, protocols=None):
        """Load"""
        if self.protocols and not force:
            return

        if not ignore_cache and os.path.exists(self.CACHEFILE):
            with open(self.CACHEFILE, 'rb') as f:
                version = cPickle.load(f)
                if version == self.CACHEFILE_VERSION:
                    self.protocols, self.fields = cPickle.load(f)
                    return
            logger.info("Cache file version mistmatch")

        cmd = ['tshark', '-G', 'fields']

        logger.info('subprocess: %s' % ' '.join(cmd))
        proc = subprocess.Popen(cmd, stdout=subprocess.PIPE)

        self.protocols = {}
        self.fields = {}
        while proc.poll() is None:
            line = proc.stdout.readline().rstrip()
            if not line:
                continue
            fields = line.split('\t')
            if fields[0] == 'P':
                (t, desc, name) = fields[:3]
                if protocols is not None and name not in protocols:
                    continue
                self.protocols[name] = desc
            elif fields[0] == 'F':
                (t, desc, name, datatype, protocol) = fields[:5]
                if protocols is not None and protocol not in protocols:
                    continue
                self.fields[name] = TSharkField(name, desc, datatype, protocol)

        with open(self.CACHEFILE, 'wb', 2) as f:
            cPickle.dump(self.CACHEFILE_VERSION, f)
            cPickle.dump([self.protocols, self.fields], f)

    def find(self, name=None, name_re=None,
             desc=None, desc_re=None,
             protocol=None, protocol_re=None):
        fields = []
        for field in self.fields.values():
            if ( (name is not None and name != field.name) or
                 (name_re is not None and not re.search(name_re, field.name)) or
                 (desc is not None and desc != field.desc) or
                 (desc_re is not None and not re.search(desc_re, field.desc)) or
                 (protocol is not None and protocol != field.protocol) or
                 (protocol_re is not None and
                  not re.search(protocol_re, field.protocol))):
                continue
            fields.append(field)
        return fields



# Copyright (c) 2015 Riverbed Technology, Inc.
#
# This software is licensed under the terms and conditions of the MIT License
# accompanying the software ("License").  This software is distributed "AS IS"
# as set forth in the License.

"""
This file defines a single report of multiple tables and widgets.

The typical structure is as follows:

    report = Report.create('Wireshark Report')
    report.add_section()

    table = SomeTable.create(name, table_options...)
    table.add_column(name, column_options...)
    table.add_column(name, column_options...)
    table.add_column(name, column_options...)

    report.add_widget(c3.TimeSeriesWidget, table, name, width=12)

See the documeantion or sample plugin for more details
"""

from steelscript.appfwk.apps.report.models import Report, Section
from steelscript.appfwk.apps.datasource.models import Column
import steelscript.appfwk.apps.report.modules.c3 as c3
import steelscript.appfwk.apps.report.modules.tables as tables

from steelscript.wireshark.appfwk.datasources.wireshark_source \
    import WiresharkColumn, WiresharkTable, WiresharkInfoTable

######################################################################
#
# PCAP analysis
#

report = Report(title="PCAP Analysis", position=10)
report.save()

report.add_section()

#
# Table: Pcap info
#

table = WiresharkInfoTable.create('pcap-info')

table.add_column('Attribute', datatype='string', iskey=True)
table.add_column('Value', datatype='string')

report.add_widget(tables.TableWidget, table, 'PCAP Info', width=12, height=200)

#
# Table: Process Pcap files
#

table = WiresharkTable.create('pcap', resample=True,
                              resolution='1m', resolutions=['1s', '1m'])

table.add_column('pkttime', datatype=Column.DATATYPE_TIME, iskey=True,
                 field='frame.time_epoch')
table.add_column('iplen', field='ip.len')

table.add_column('iplen-bits', synthetic=True,
                 compute_expression='8*{iplen}',
                 resample_operation='sum')
table.add_column('max-iplen', synthetic=True,
                 compute_expression='{iplen}',
                 resample_operation='max')
table.add_column('min-iplen', synthetic=True,
                 compute_expression='{iplen}',
                 resample_operation='min')
table.add_column('limit_100', synthetic=True,
                 compute_expression='100',
                 resample_operation='min')

# Compute 95th percentile
table.add_column('iplen_95', synthetic=True, label="95%",
                 compute_expression='{iplen}.quantile(0.95)',
                 compute_post_resample=True)

# Compute 80th percentile
table.add_column('iplen_80', synthetic=True, label="80%",
                 compute_expression='{iplen}.quantile(0.80)',
                 compute_post_resample=True)

# Compute rolling average (EWMA algo)
table.add_column('iplen_ewma', synthetic=True, label="Moving Avg",
                 compute_expression='pandas.stats.moments.ewma({iplen}, span=20)',
                 compute_post_resample=True)


report.add_widget(c3.TimeSeriesWidget, table, "IP Bytes over Time",
                  width=12, height=400,
                  valuecols=['iplen', 'iplen_95', 'iplen_80', 'iplen_ewma'])



# Copyright (c) 2015 Riverbed Technology, Inc.
#
# This software is licensed under the terms and conditions of the MIT License
# accompanying the software ("License").  This software is distributed "AS IS"
# as set forth in the License.


class WiresharkException(Exception):
    pass


class InvalidField(WiresharkException):
    def __init__(self, name):
        self.name = name

    def __str__(self):
        return "Invalid Wireshark field: %s" % self.name



# Copyright (c) 2015 Riverbed Technology, Inc.
#
# This software is licensed under the terms and conditions of the MIT License
# accompanying the software ("License").  This software is distributed "AS IS"
# as set forth in the License.

"""
A data source for querying data from pcap files via tshark
"""

import logging
import os
import math
import copy
import pandas
import shutil
import subprocess
import multiprocessing

from django import forms
from django.conf import settings
from django.forms.widgets import FileInput

from steelscript.wireshark.core.pcap import PcapFile

from steelscript.appfwk.apps.datasource.models \
    import DatasourceTable, TableField, Column, TableQueryBase
from steelscript.appfwk.apps.datasource.forms \
    import FileSelectField, fields_add_resolution, fields_add_time_selection
from steelscript.appfwk.apps.jobs import QueryComplete, QueryContinue, Job
from steelscript.appfwk.apps.datasource.modules.analysis import \
    AnalysisQuery, AnalysisException, AnalysisTable
from steelscript.appfwk.apps.datasource.models import Table

logger = logging.getLogger(__name__)


SPLIT_DIR = '/tmp/split_pcaps'


class WiresharkColumn(Column):
    class Meta:
        proxy = True

    COLUMN_OPTIONS = {'field': None,
                      'operation': 'sum'}


def fields_add_pcapfile(obj, keyword='pcapfilename',
                        label='PCAP File', initial=None,
                        astextfield=False):
    """Add a PCAP file selection field.

    :param bool astextfield: If True, use a text field instead of a
        file selection field.  The text value is interpreted as
        a file on the server.

    """

    kwargs = {}
    if not astextfield:
        kwargs['field_cls'] = FileSelectField
        kwargs['field_kwargs'] = {'widget': FileInput}

    field = TableField(keyword=keyword,
                       label=label,
                       **kwargs)
    field.save()
    obj.fields.add(field)


def fields_add_filterexpr(obj,
                          keyword='wireshark_filterexpr',
                          initial=None
                          ):
    field = (TableField
             (keyword=keyword,
              label='WireShark Filter Expression',
              help_text=('Traffic expression using WireShark Display '
                         'Filter syntax'),
              initial=initial,
              required=False))
    field.save()
    obj.fields.add(field)


class WiresharkTable(DatasourceTable):

    class Meta:
        proxy = True

    # When a custom column is used, it must be linked
    _column_class = 'WiresharkColumn'
    _query_class = 'WiresharkQuery'

    TABLE_OPTIONS = {'show_upload': True,
                     'show_entire_pcap': True}
    FIELD_OPTIONS = {'resolution': '1m',
                     'resolutions': ('1s', '1m', '15min', '1h'),
                     'pcapfile_astextfield': False}

    def post_process_table(self, field_options):
        #
        # Add criteria fields that are required by this table
        #
        if self.options.show_entire_pcap:
            TableField.create(keyword='entire_pcap', obj=self,
                              field_cls=forms.BooleanField,
                              label='Entire PCAP',
                              initial=True,
                              required=False)

        fields_add_time_selection(self, show_start=True, show_end=True,
                                  show_duration=False)
        fields_add_resolution(obj=self,
                              initial=field_options['resolution'],
                              resolutions=field_options['resolutions'])

        if self.options.show_upload:
            fields_add_pcapfile(
                obj=self, astextfield=field_options['pcapfile_astextfield'])
        fields_add_filterexpr(obj=self)


class WiresharkQuery(TableQueryBase):

    def run(self):
        criteria = self.job.criteria

        table = self.table
        columns = table.get_columns(synthetic=False)

        pcapfilename = criteria.pcapfilename

        if not pcapfilename:
            raise ValueError("No pcap file specified")
        elif not os.path.exists(pcapfilename):
            raise ValueError("No such file: %s" % pcapfilename)

        if not hasattr(settings, 'TSHARK_PATH'):
            raise ValueError('Please set local_settings.TSHARK_PATH '
                             'to the proper path to the tshark executable')

        pcapfile = PcapFile(pcapfilename)

        fieldnames = []
        basecolnames = []  # list of colummns
        # dict by field name of the base (or first) column to use this field
        fields = {}
        for tc in columns:
            tc_options = tc.options
            if tc_options.field in fields.keys():
                # Asking for the same field name twice doesn't work, but
                # is useful when aggregating and choosing a different operation
                # like "min", or "max".  Will populate these columns later
                continue
            fields[tc_options.field] = tc.name
            fieldnames.append(tc_options.field)
            basecolnames.append(tc.name)

        if criteria.entire_pcap:
            starttime = None
            endtime = None
        else:
            starttime = criteria.starttime
            endtime = criteria.endtime

        data = pcapfile.query(
            fieldnames,
            starttime=starttime,
            endtime=endtime,
            filterexpr=criteria.wireshark_filterexpr,
            use_tshark_fields=True)

        if len(data) == 0:
            self.data = None
            return True

        df = pandas.DataFrame(data, columns=basecolnames)
        # At this point we have a dataframe with the one column for each
        # unique field (the first column to reference the field)

        if table.rows > 0:
            df = df[:table.rows]

        logger.info("Data returned (first 3 rows...):\n%s", df[:3])

        # Convert the data into the right format
        for tc in columns:
            if tc.name not in basecolnames:
                continue
            tc_options = tc.options
            if tc.datatype == "time":
                df[tc.name] = pandas.DatetimeIndex(df[tc.name])

        colnames = [col.name for col in columns]
        self.data = df.ix[:, colnames].values.tolist()

        return True


class WiresharkInfoTable(DatasourceTable):

    class Meta:
        proxy = True

    _query_class = 'WiresharkInfoQuery'

    TABLE_OPTIONS = {}
    FIELD_OPTIONS = {}

    def post_process_table(self, field_options):
        fields_add_pcapfile(obj=self)


class WiresharkInfoQuery(TableQueryBase):

    def run(self):
        criteria = self.job.criteria

        pcapfilename = criteria.pcapfilename

        if not pcapfilename:
            raise ValueError("No pcap file specified")
        elif not os.path.exists(pcapfilename):
            raise ValueError("No such file: %s" % pcapfilename)

        if not hasattr(settings, 'TSHARK_PATH'):
            raise ValueError('Please set local_settings.TSHARK_PATH '
                             'to the proper path to the tshark executable')

        pcapfile = PcapFile(pcapfilename)
        pcapfile.info()
        self.data = [['Start time', str(pcapfile.starttime)],
                     ['End time', str(pcapfile.endtime)],
                     ['Number of packets', pcapfile.numpackets]]
        return True


class WiresharkPcapTable(AnalysisTable):
    """This table processes downloaded or uploaded PCAP files.
    For the downloaded case, this table looks at the dependant
    table to determine where to find the PCAP file to analyze.
    It also supports uploading a PCAP file to analyze. When
    processing the PCAP file, this table might split the file
    to optimize performance based on the split_threshold table
    option.
    """
    class Meta:
        proxy = True

    _column_class = 'WiresharkColumn'
    _query_class = 'WiresharkPcapQuery'

    TABLE_OPTIONS = {'split_threshold': 0,
                     }

    FIELD_OPTIONS = {'resolution': '1s',
                     'resolutions': ('1s', '1m', '15min', '1h')}


class WiresharkPcapQuery(AnalysisQuery):

    def split_pcap(self):
        cpu_num = multiprocessing.cpu_count()
        per_file = int(math.ceil(self.pkt_num/cpu_num))

        if not os.path.exists(SPLIT_DIR):
            os.mkdir(SPLIT_DIR)
        os.mkdir(self.output_dir)

        cmd = 'editcap -c %s %s %s/' % (per_file, self.filename,
                                        self.output_dir)
        subprocess.Popen(cmd, shell=True).wait()

    @property
    def file_handle(self):
        """Return the basename of the path without extension.
        If filename is 'a/b/c.d', return 'c'.
        """
        return os.path.basename(self.filename).rsplit('.', 1)[0]

    def analyze(self, jobs=None):

        criteria = self.job.criteria

        if jobs:
            job = jobs.values()[0]
            if job.status == Job.ERROR:
                raise AnalysisException("%s for getting pcap file failed: %s"
                                        % (job, job.message))
            criteria.entire_pcap = True
            self.filename = job.data()['filename'][0]
        else:
            self.filename = criteria.pcapfilename

        pcap = PcapFile(self.filename)

        try:
            pcap_info = pcap.info()
        except ValueError:
            raise AnalysisException("No packets in %s" % self.filename)

        logger.debug("%s: File info %s" % (self.__class__.__name__, pcap_info))

        self.pkt_num = int(pcap_info['Number of packets'])

        min_pkt_num = self.table.options.split_threshold

        wt = Table.from_ref(self.table.options.related_tables['wireshark'])

        depjobs = {}
        if self.pkt_num < min_pkt_num:
            # No need to split the pcap file
            criteria.pcapfilename = self.filename
            criteria.entire_pcap = True
            job = Job.create(table=wt, criteria=criteria,
                             update_progress=False, parent=self.job)

            depjobs[job.id] = job

            logger.debug("%s starting single job" % self.__class__.__name__)
            return QueryContinue(self.collect, depjobs)

        self.output_dir = os.path.join(SPLIT_DIR, self.file_handle)
        self.split_pcap()

        split_files = os.listdir(self.output_dir)

        if not split_files:
            raise AnalysisException('No pcap file found after splitting %s'
                                    % self.filename)

        for split in split_files:
            # use wireshark table
            ws_criteria = copy.copy(criteria)
            ws_criteria.pcapfilename = os.path.join(self.output_dir, split)

            # for ease of removing the split directory in collect func
            ws_criteria.output_dir = self.output_dir

            job = Job.create(table=wt, criteria=ws_criteria,
                             update_progress=False, parent=self.job)

            depjobs[job.id] = job

        logger.debug("%s starting multiple jobs" % self.__class__.__name__)

        return QueryContinue(self.collect, jobs=depjobs)

    def collect(self, jobs=None):
        dfs = []

        # Removing the temporary split directory if it exists
        output_dir = getattr(jobs.values()[0].criteria, 'output_dir', None)
        if output_dir and os.path.exists(output_dir):
            shutil.rmtree(output_dir)

        for jid, job in jobs.iteritems():
            if job.status == Job.ERROR:
                raise AnalysisException("%s for pcap file %s failed: %s"
                                        % (job, job.criteria.pcapfilename,
                                           job.message))
            subdf = job.data()
            if subdf is None:
                continue
            dfs.append(subdf)

        if not dfs:
            logger.debug("%s: no data is collected" % self.__class__.__name__)
            return QueryComplete(None)

        df = pandas.concat(dfs, ignore_index=True)

        logger.debug("%s: Query ended." % self.__class__.__name__)

        return QueryComplete(df)



# Copyright (c) 2015 Riverbed Technology, Inc.
#
# This software is licensed under the terms and conditions of the MIT License
# accompanying the software ("License").  This software is distributed "AS IS"
# as set forth in the License.

"""
This file defines a device associated with this plugin
"""

#
# new_device_instance
#
# Implement this method to return a device instance to the target
# host/port using the request authentication credentials.
#
# This function must return an object that can be used to
# interact with this device.  The object returned can be of
# any type, and the methods used to interact with the actual
# device are implementation specific.
#
# This method is called as a consequence of:
#    dev = DeviceManager.get_device(id)
#
# The above is normally invoked from a datasource TableQuery.run()
# method just before interacting with the device to perform
# a query.
#
# The device associated with id is looked up in the Device database
# table and the new_device_instance() method is invoked for that
# device's module.
#
# The returned object is cached by the device id and returned
# on subsequent calls to get_device() for the same id.  The object
# is then returned to the caller.
#
# This method should *only* be used if the resulting device object
# is thread-safe and can handle interactions from multiple jobs
# simultaneously.
#

def new_device_instance(host, port, auth):
    # This function must be implemented and should return
    # a device object.
    return None



#!/usr/bin/env python

# Copyright (c) 2015 Riverbed Technology, Inc.
#
# This software is licensed under the terms and conditions of the MIT License
# accompanying the software ("License").  This software is distributed "AS IS"
# as set forth in the License.


"""
This example script shows how to get basic details about a pcap file.
"""

import os

from steelscript.common.app import Application
from steelscript.common.datautils import Formatter
from steelscript.wireshark.core.pcap import PcapFile


class PcapInfo(Application):
    """Simple PCAP query application."""

    def add_positional_args(self):
        self.add_positional_arg('pcap_path', 'File path to pcap file')

    def validate_args(self):
        super(PcapInfo, self).validate_args()

        path = self.options.pcap_path

        if path is None or not os.path.exists(path):
            self.parser.error('Must pass absolute path to PCAP file.')

    def main(self):
        pcap = PcapFile(self.options.pcap_path)
        info = pcap.info()

        Formatter.print_table([(k, unicode(v, errors='replace')) for k, v
                               in info.iteritems()],
                              headers=['Key', 'Value'])

if __name__ == '__main__':
    PcapInfo().run()



#!/usr/bin/env python

# Copyright (c) 2015 Riverbed Technology, Inc.
#
# This software is licensed under the terms and conditions of the MIT License
# accompanying the software ("License").  This software is distributed "AS IS"
# as set forth in the License.


"""
This example script shows how to utilize the query mechanism to get
basic info about a pcap file.
"""

import os

from steelscript.common.app import Application
from steelscript.common.datautils import Formatter
from steelscript.wireshark.core.pcap import PcapFile


class PcapInfo(Application):
    """Simple PCAP Info application."""

    def add_positional_args(self):
        self.add_positional_arg('pcap_path', 'File path to pcap file')

    def add_options(self, parser):
        super(PcapInfo, self).add_options(parser)
        parser.add_option('-c', '--columns',
                          help='Comma-separated list of Wireshark columns '
                               'to return. Defaults to a basic set of '
                               'frame time and data length per packet.',
                          default='frame.time_epoch,ip.len')
        parser.add_option('-m', '--max-rows', default=50,
                          help='Max number of rows to print out.  Defaults '
                               'to 50.')
        parser.add_option('-j', '--join', dest='join', default='FULL',
                          help='The type of join to be used to construct the '
                               'table. Possible values: FULL, INNER')

    def validate_args(self):
        super(PcapInfo, self).validate_args()

        path = self.options.pcap_path
        join = self.options.join

        if path is None or not os.path.exists(path):
            self.parser.error('Must pass absolute path to PCAP file.')
        if join is not None and join not in ['FULL', 'INNER']:
            self.parser.error('Join must be either FULL or INNER.')

    def main(self):
        columns = self.options.columns.split(',')
        pcap = PcapFile(self.options.pcap_path)
        data = pcap.query(columns)

        if self.options.join == 'INNER':
            data = [row for row in data if None not in row]

        if not data:
            print('No rows found matching your input')

        max_rows = int(self.options.max_rows)
        data_out = data[:max_rows]

        Formatter.print_table(data_out, headers=columns)

if __name__ == '__main__':
    PcapInfo().run()



# Copyright (c) 2015 Riverbed Technology, Inc.
#
# This software is licensed under the terms and conditions of the MIT License
# accompanying the software ("License").  This software is distributed "AS IS"
# as set forth in the License.

import pkg_resources

from steelscript.appfwk.apps.plugins import Plugin as AppsPlugin


class Plugin(AppsPlugin):
    title = 'Wireshark plugin'
    description = 'Extract metrics from pcap files'
    version = pkg_resources.get_distribution('steelscript.wireshark').version
    author = 'Riverbed Technology'

    enabled = True
    can_disable = True

    devices = ['devices']
    datasources = ['datasources']
    reports = ['reports']



# Copyright (c) 2015 Riverbed Technology, Inc.
#
# This software is licensed under the terms and conditions of the MIT License
# accompanying the software ("License").  This software is distributed "AS IS"
# as set forth in the License.

"""
steelscript.wireshark
====================
Extract metrics from pcap files

"""
from glob import glob
from setuptools import setup, find_packages
from gitpy_versioning import get_version

setup_args = {
    'name':                'steelscript.wireshark',
    'namespace_packages':  ['steelscript'],
    'version':             get_version(),

    # Update the following as needed
    'author':              'Riverbed Technology',
    'author_email':        'eng-github@riverbed.com',
    'url':                 'http://pythonhosted.org/steelscript',
    'license':             'MIT',
    'description':         'Extract metrics from pcap files',

    'long_description': """SteelScript for Wireshark
=========================

SteelScript is a collection of libraries and scripts in Python and JavaScript for
interacting with Riverbed Technology devices.

For a complete guide to installation, see:

http://pythonhosted.org/steelscript/
    """,

    'packages': find_packages(exclude=('gitpy_versioning',)),
    'zip_safe': False,

    'install_requires': (
        'steelscript',
        'python-dateutil',
        'tzlocal',
    ),

    'extras_require': None,
    'test_suite': '',
    'include_package_data': True,

    'entry_points': {
        'portal.plugins': [
            'wireshark = steelscript.wireshark.appfwk.plugin:Plugin'
        ],
    },

    'data_files': (
        ('share/doc/steelscript/docs/wireshark', glob('docs/*')),
        ('share/doc/steelscript/examples/wireshark', glob('examples/*')),
    ),

    'classifiers': (
        'Framework :: Django',
        'Intended Audience :: Developers',
        'Intended Audience :: System Administrators',
        'Intended Audience :: Information Technology',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 2.7',
        'Topic :: Software Development',
        'Topic :: System :: Networking',
    ),
}

setup(**setup_args)
